/**
 ********************************************************************
 * @file    test_payload_cam_emu.c
 * @version V2.0.0
 * @date    2019/07/01
 * @brief
 *
 * @copyright (c) 2018-2019 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 *********************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "test_payload_cam_emu.h"
#include <psdk_logger.h>
#include <psdk_gimbal.h>
#include "psdk_payload_camera.h"
#include "psdk_platform.h"
#include "utils/util_misc.h"
#include "psdk_aircraft_info.h"
#include "psdk_xport.h"
#include "gimbal_emu/test_payload_gimbal_emu.h"
#include "math.h"

/* Private constants ---------------------------------------------------------*/
#define SDCARD_TOTAL_SPACE_IN_MB                (32 * 1024)
#define SDCARD_PER_PHOTO_SPACE_IN_MB            (4)
#define SDCARD_PER_SECONDS_RECORD_SPACE_IN_MB   (2)
#define ZOOM_OPTICAL_FOCAL_MAX_LENGTH           (300)
#define ZOOM_OPTICAL_FOCAL_MIN_LENGTH           (10)
#define ZOOM_OPTICAL_FOCAL_LENGTH_STEP          (10)
#define ZOOM_OPTICAL_FOCAL_LENGTH_CTRL_STEP     (5)
#define ZOOM_DIGITAL_BASE_FACTOR                (1.0)
#define FOCUS_MAX_RINGVALUE                     (1000)
#define FOCUS_MID_RINGVALUE                     (500)
#define IMAGE_SENSOR_X_SIZE                     (88.0f) // unit: 0.1mm
#define IMAGE_SENSOR_Y_SIZE                     (66.0f) // unit: 0.1mm
#define CENTER_POINT_IN_SCREEN_X_VALUE          (0.5f)
#define CENTER_POINT_IN_SCREEN_Y_VALUE          (0.5f)
#define TAP_ZOOM_DURATION                       (2000) // unit: ms
#define PAYLOAD_CAMERA_EMU_TASK_STACK_SIZE      (2048)
#define PI                                      (3.14159265358979323846f)
#define INTERVAL_PHOTOGRAPH_ALWAYS_COUNT        (255)

/* Private types -------------------------------------------------------------*/
typedef struct {
    E_PsdkGimbalRotationMode rotationMode;
    T_PsdkGimbalRotationProperty rotationProperty;
    T_PsdkAttitude3d rotationValue;
} T_TestCameraGimbalRotationArgument;

/* Private variables ---------------------------------------------------------*/
static T_PsdkCameraCommonHandler s_commonHandler;
static T_PsdkCameraExposureMeteringHandler s_exposureMeteringHandler;
static T_PsdkCameraFocusHandler s_focusHandler;
static T_PsdkCameraDigitalZoomHandler s_digitalZoomHandler;
static T_PsdkCameraOpticalZoomHandler s_opticalZoomHandler;
static T_PsdkCameraTapZoomHandler s_tapZoomHandler;

static T_PsdkTaskHandle s_userCameraThread;

static T_PsdkCameraSystemState s_cameraState;
static E_PsdkCameraShootPhotoMode s_cameraShootPhotoMode = PSDK_CAMERA_SHOOT_PHOTO_MODE_SINGLE;
static E_PsdkCameraBurstCount s_cameraBurstCount = PSDK_CAMERA_BURST_COUNT_2;
static T_PsdkCameraPhotoTimeIntervalSettings s_cameraPhotoTimeIntervalSettings = {0};
static T_PsdkCameraSDCardState s_cameraSDCardState = {0};
static T_PsdkMutexHandle s_commonMutex = {0};

static E_PsdkCameraMeteringMode s_cameraMeteringMode = PSDK_CAMERA_METERING_MODE_CENTER;
static T_PsdkCameraSpotMeteringTarget s_cameraSpotMeteringTarget = {0};

static E_PsdkCameraFocusMode s_cameraFocusMode = PSDK_CAMERA_FOCUS_MODE_AUTO;
static T_PsdkCameraPointInScreen s_cameraFocusTarget = {0};
static uint32_t s_cameraFocusRingValue = FOCUS_MID_RINGVALUE;
static T_PsdkCameraFocusAssistantSettings s_cameraFocusAssistantSettings = {0};

static E_PsdkCameraZoomDirection s_cameraZoomDirection = PSDK_CAMERA_ZOOM_DIRECTION_OUT;
static E_PsdkCameraZoomSpeed s_cameraZoomSpeed = PSDK_CAMERA_ZOOM_SPEED_NORMAL;
static uint32_t s_cameraOpticalZoomFocalLength = ZOOM_OPTICAL_FOCAL_MIN_LENGTH;
static psdk_f32_t s_cameraDigitalZoomFactor = ZOOM_DIGITAL_BASE_FACTOR;
static bool s_isStartContinuousOpticalZoom = false;
static T_PsdkMutexHandle s_zoomMutex = {0};

static bool s_isTapZoomEnabled = false;
static T_PsdkCameraTapZoomState s_cameraTapZoomState = {0};
static uint8_t s_tapZoomMultiplier = 1;
static uint32_t s_tapZoomStartTime = 0;
static bool s_isTapZooming = false;
static T_TestCameraGimbalRotationArgument s_tapZoomNewestGimbalRotationArgument = {0};
static uint32_t s_tapZoomNewestTargetHybridFocalLength = 0; // unit: 0.1mm
static T_PsdkSemHandle s_tapZoomSem = NULL;
static T_PsdkMutexHandle s_tapZoomMutex = NULL;

/* Private functions declaration ---------------------------------------------*/
static T_PsdkReturnCode GetSystemState(T_PsdkCameraSystemState *systemState);
static T_PsdkReturnCode SetMode(E_PsdkCameraMode mode);
static T_PsdkReturnCode GetMode(E_PsdkCameraMode *mode);
static T_PsdkReturnCode StartRecordVideo(void);
static T_PsdkReturnCode StopRecordVideo(void);
static T_PsdkReturnCode StartShootPhoto(void);
static T_PsdkReturnCode StopShootPhoto(void);
static T_PsdkReturnCode SetShootPhotoMode(E_PsdkCameraShootPhotoMode mode);
static T_PsdkReturnCode GetShootPhotoMode(E_PsdkCameraShootPhotoMode *mode);
static T_PsdkReturnCode SetPhotoBurstCount(E_PsdkCameraBurstCount burstCount);
static T_PsdkReturnCode GetPhotoBurstCount(E_PsdkCameraBurstCount *burstCount);
static T_PsdkReturnCode SetPhotoTimeIntervalSettings(T_PsdkCameraPhotoTimeIntervalSettings settings);
static T_PsdkReturnCode GetPhotoTimeIntervalSettings(T_PsdkCameraPhotoTimeIntervalSettings *settings);
static T_PsdkReturnCode GetSDCardState(T_PsdkCameraSDCardState *sdCardState);
static T_PsdkReturnCode FormatSDCard(void);

static T_PsdkReturnCode SetMeteringMode(E_PsdkCameraMeteringMode mode);
static T_PsdkReturnCode GetMeteringMode(E_PsdkCameraMeteringMode *mode);
static T_PsdkReturnCode SetSpotMeteringTarget(T_PsdkCameraSpotMeteringTarget target);
static T_PsdkReturnCode GetSpotMeteringTarget(T_PsdkCameraSpotMeteringTarget *target);

static T_PsdkReturnCode SetFocusMode(E_PsdkCameraFocusMode mode);
static T_PsdkReturnCode GetFocusMode(E_PsdkCameraFocusMode *mode);
static T_PsdkReturnCode SetFocusTarget(T_PsdkCameraPointInScreen target);
static T_PsdkReturnCode GetFocusTarget(T_PsdkCameraPointInScreen *target);
static T_PsdkReturnCode SetFocusAssistantSettings(T_PsdkCameraFocusAssistantSettings settings);
static T_PsdkReturnCode GetFocusAssistantSettings(T_PsdkCameraFocusAssistantSettings *settings);
static T_PsdkReturnCode SetFocusRingValue(uint32_t value);
static T_PsdkReturnCode GetFocusRingValue(uint32_t *value);
static T_PsdkReturnCode GetFocusRingValueUpperBound(uint32_t *value);

static T_PsdkReturnCode SetDigitalZoomFactor(psdk_f32_t factor);
static T_PsdkReturnCode SetOpticalZoomFocalLength(uint32_t focalLength);
static T_PsdkReturnCode GetOpticalZoomFocalLength(uint32_t *focalLength);
static T_PsdkReturnCode GetOpticalZoomSpec(T_PsdkCameraOpticalZoomSpec *spec);
static T_PsdkReturnCode StartContinuousOpticalZoom(E_PsdkCameraZoomDirection direction, E_PsdkCameraZoomSpeed speed);
static T_PsdkReturnCode StopContinuousOpticalZoom(void);
static T_PsdkReturnCode GetTapZoomState(T_PsdkCameraTapZoomState *state);
static T_PsdkReturnCode SetTapZoomEnabled(bool enabledFlag);
static T_PsdkReturnCode GetTapZoomEnabled(bool *enabledFlag);
static T_PsdkReturnCode SetTapZoomMultiplier(uint8_t multiplier);
static T_PsdkReturnCode GetTapZoomMultiplier(uint8_t *multiplier);
static T_PsdkReturnCode TapZoomAtTarget(T_PsdkCameraPointInScreen target);
static T_PsdkReturnCode PsdkTest_CameraHybridZoom(uint32_t focalLength);
static T_PsdkReturnCode PsdkTest_CameraRotationGimbal(T_TestCameraGimbalRotationArgument gimbalRotationArgument);

static void *UserCamera_Task(void *arg);

/* Exported functions definition ---------------------------------------------*/

/* Private functions definition-----------------------------------------------*/
static T_PsdkReturnCode GetSystemState(T_PsdkCameraSystemState *systemState)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    *systemState = s_cameraState;

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetMode(E_PsdkCameraMode mode)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    s_cameraState.cameraMode = mode;
    PsdkLogger_UserLogDebug("set camera mode:%d", mode);

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetMode(E_PsdkCameraMode *mode)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    *mode = s_cameraState.cameraMode;

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode StartRecordVideo(void)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    s_cameraState.isRecording = true;
    PsdkLogger_UserLogDebug("start record video");

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode StopRecordVideo(void)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    s_cameraState.isRecording = false;
    s_cameraState.currentVideoRecordingTimeInSeconds = 0;
    PsdkLogger_UserLogDebug("stop record video");

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode StartShootPhoto(void)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    PsdkLogger_UserLogDebug("start shoot photo");
    s_cameraState.isStoring = true;

    if (s_cameraShootPhotoMode == PSDK_CAMERA_SHOOT_PHOTO_MODE_SINGLE) {
        s_cameraState.shootingState = PSDK_CAMERA_SHOOTING_SINGLE_PHOTO;
    } else if (s_cameraShootPhotoMode == PSDK_CAMERA_SHOOT_PHOTO_MODE_BURST) {
        s_cameraState.shootingState = PSDK_CAMERA_SHOOTING_BURST_PHOTO;
    } else if (s_cameraShootPhotoMode == PSDK_CAMERA_SHOOT_PHOTO_MODE_INTERVAL) {
        s_cameraState.shootingState = PSDK_CAMERA_SHOOTING_INTERVAL_PHOTO;
        s_cameraState.isShootingIntervalStart = true;
        s_cameraState.currentPhotoShootingIntervalTimeInSeconds = s_cameraPhotoTimeIntervalSettings.timeIntervalSeconds;
    }

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode StopShootPhoto(void)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    PsdkLogger_UserLogDebug("stop shoot photo");
    s_cameraState.shootingState = PSDK_CAMERA_SHOOTING_PHOTO_IDLE;
    s_cameraState.isStoring = false;
    s_cameraState.isShootingIntervalStart = false;

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetShootPhotoMode(E_PsdkCameraShootPhotoMode mode)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    s_cameraShootPhotoMode = mode;
    PsdkLogger_UserLogDebug("set shoot photo mode:%d", mode);

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetShootPhotoMode(E_PsdkCameraShootPhotoMode *mode)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    *mode = s_cameraShootPhotoMode;

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);\
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetPhotoBurstCount(E_PsdkCameraBurstCount burstCount)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    s_cameraBurstCount = burstCount;
    PsdkLogger_UserLogDebug("set photo burst count:%d", burstCount);

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetPhotoBurstCount(E_PsdkCameraBurstCount *burstCount)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    *burstCount = s_cameraBurstCount;

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetPhotoTimeIntervalSettings(T_PsdkCameraPhotoTimeIntervalSettings settings)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    s_cameraPhotoTimeIntervalSettings.captureCount = settings.captureCount;
    s_cameraPhotoTimeIntervalSettings.timeIntervalSeconds = settings.timeIntervalSeconds;
    PsdkLogger_UserLogDebug("set photo interval settings count:%d seconds:%d", settings.captureCount,
                            settings.timeIntervalSeconds);
    s_cameraState.currentPhotoShootingIntervalCount = settings.captureCount;

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetPhotoTimeIntervalSettings(T_PsdkCameraPhotoTimeIntervalSettings *settings)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    memcpy(settings, &s_cameraPhotoTimeIntervalSettings, sizeof(T_PsdkCameraPhotoTimeIntervalSettings));

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetSDCardState(T_PsdkCameraSDCardState *sdCardState)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    memcpy(sdCardState, &s_cameraSDCardState, sizeof(T_PsdkCameraSDCardState));

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode FormatSDCard(void)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    PsdkLogger_UserLogDebug("format sdcard");

    memset(&s_cameraSDCardState, 0, sizeof(T_PsdkCameraSDCardState));
    s_cameraSDCardState.isInserted = true;
    s_cameraSDCardState.totalSpaceInMB = SDCARD_TOTAL_SPACE_IN_MB;
    s_cameraSDCardState.remainSpaceInMB = SDCARD_TOTAL_SPACE_IN_MB;
    s_cameraSDCardState.availableCaptureCount = SDCARD_TOTAL_SPACE_IN_MB / SDCARD_PER_PHOTO_SPACE_IN_MB;
    s_cameraSDCardState.availableRecordingTimeInSeconds =
        SDCARD_TOTAL_SPACE_IN_MB / SDCARD_PER_SECONDS_RECORD_SPACE_IN_MB;

    psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetMeteringMode(E_PsdkCameraMeteringMode mode)
{
    PsdkLogger_UserLogDebug("set metering mode:%d", mode);
    s_cameraMeteringMode = mode;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetMeteringMode(E_PsdkCameraMeteringMode *mode)
{
    *mode = s_cameraMeteringMode;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetSpotMeteringTarget(T_PsdkCameraSpotMeteringTarget target)
{
    PsdkLogger_UserLogDebug("set spot metering area col:%d row:%d", target.col, target.row);
    memcpy(&s_cameraSpotMeteringTarget, &target, sizeof(T_PsdkCameraSpotMeteringTarget));

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetSpotMeteringTarget(T_PsdkCameraSpotMeteringTarget *target)
{
    memcpy(target, &s_cameraSpotMeteringTarget, sizeof(T_PsdkCameraSpotMeteringTarget));

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetFocusMode(E_PsdkCameraFocusMode mode)
{
    PsdkLogger_UserLogDebug("set focus mode:%d", mode);
    s_cameraFocusMode = mode;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetFocusMode(E_PsdkCameraFocusMode *mode)
{
    *mode = s_cameraFocusMode;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetFocusTarget(T_PsdkCameraPointInScreen target)
{
    PsdkLogger_UserLogDebug("set focus target x:%.2f y:%.2f", target.focusX, target.focusY);
    memcpy(&s_cameraFocusTarget, &target, sizeof(T_PsdkCameraPointInScreen));

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetFocusTarget(T_PsdkCameraPointInScreen *target)
{
    memcpy(target, &s_cameraFocusTarget, sizeof(T_PsdkCameraPointInScreen));

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetFocusAssistantSettings(T_PsdkCameraFocusAssistantSettings settings)
{
    PsdkLogger_UserLogDebug("set focus assistant setting MF:%d AF:%d", settings.isEnabledMF, settings.isEnabledAF);
    memcpy(&s_cameraFocusAssistantSettings, &settings, sizeof(T_PsdkCameraFocusAssistantSettings));

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetFocusAssistantSettings(T_PsdkCameraFocusAssistantSettings *settings)
{
    memcpy(settings, &s_cameraFocusAssistantSettings, sizeof(T_PsdkCameraFocusAssistantSettings));

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetFocusRingValue(uint32_t value)
{
    PsdkLogger_UserLogDebug("set focus ring value:%d", value);
    s_cameraFocusRingValue = value;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetFocusRingValue(uint32_t *value)
{
    *value = s_cameraFocusRingValue;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetFocusRingValueUpperBound(uint32_t *value)
{
    *value = FOCUS_MAX_RINGVALUE;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetDigitalZoomFactor(psdk_f32_t factor)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    PsdkLogger_UserLogDebug("set digital zoom factor:%.2f", factor);
    s_cameraDigitalZoomFactor = factor;

    psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetOpticalZoomFocalLength(uint32_t focalLength)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    PsdkLogger_UserLogDebug("set optical zoom focal length:%d", focalLength);
    s_cameraOpticalZoomFocalLength = focalLength;

    psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetOpticalZoomFocalLength(uint32_t *focalLength)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    *focalLength = s_cameraOpticalZoomFocalLength;

    psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetOpticalZoomSpec(T_PsdkCameraOpticalZoomSpec *spec)
{
    spec->maxFocalLength = ZOOM_OPTICAL_FOCAL_MAX_LENGTH;
    spec->minFocalLength = ZOOM_OPTICAL_FOCAL_MIN_LENGTH;
    spec->focalLengthStep = ZOOM_OPTICAL_FOCAL_LENGTH_STEP;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode StartContinuousOpticalZoom(E_PsdkCameraZoomDirection direction, E_PsdkCameraZoomSpeed speed)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    PsdkLogger_UserLogDebug("start continuous optical zoom direction:%d speed:%d", direction, speed);
    s_isStartContinuousOpticalZoom = true;
    s_cameraZoomDirection = direction;
    s_cameraZoomSpeed = speed;

    psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode StopContinuousOpticalZoom(void)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    PsdkLogger_UserLogDebug("stop continuous optical zoom");
    s_isStartContinuousOpticalZoom = false;
    s_cameraZoomDirection = PSDK_CAMERA_ZOOM_DIRECTION_OUT;
    s_cameraZoomSpeed = PSDK_CAMERA_ZOOM_SPEED_NORMAL;

    psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetTapZoomState(T_PsdkCameraTapZoomState *state)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_tapZoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    memcpy(state, &s_cameraTapZoomState, sizeof(T_PsdkCameraTapZoomState));

    psdkStat = PsdkOsal_MutexUnlock(s_tapZoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetTapZoomEnabled(bool enabledFlag)
{
    PsdkLogger_UserLogDebug("set tap zoom enabled flag: %d.", enabledFlag);
    s_isTapZoomEnabled = enabledFlag;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetTapZoomEnabled(bool *enabledFlag)
{
    *enabledFlag = s_isTapZoomEnabled;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode SetTapZoomMultiplier(uint8_t multiplier)
{
    PsdkLogger_UserLogDebug("set tap zoom multiplier: %d.", multiplier);
    s_tapZoomMultiplier = multiplier;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode GetTapZoomMultiplier(uint8_t *multiplier)
{
    *multiplier = s_tapZoomMultiplier;

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode TapZoomAtTarget(T_PsdkCameraPointInScreen target)
{
    T_PsdkReturnCode psdkStat;
    T_PsdkReturnCode errorCode;
    E_PsdkGimbalRotationMode rotationMode;
    T_PsdkGimbalRotationProperty rotationProperty = {0};
    T_PsdkAttitude3d rotationValue = {0};
    float hybridFocalLength = 0; // unit: 0.1mm

    PsdkLogger_UserLogDebug("tap zoom at target: x %f, y %f.", target.focusX, target.focusY);

    if (s_isTapZoomEnabled != true) {
        PsdkLogger_UserLogWarn("tap zoom is not enabled.");
        return PSDK_RETURN_CODE_ERR_SYSTEM;
    }

    psdkStat = PsdkOsal_GetTimeMs(&s_tapZoomStartTime);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("get start time error: %lld.", psdkStat);
        errorCode = psdkStat;
        goto err;
    }

    rotationMode = PSDK_GIMBAL_ROTATION_MODE_RELATIVE_ANGLE;
    rotationProperty.relativeAngleRotation.actionTime = TAP_ZOOM_DURATION / 10;

    psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        errorCode = psdkStat;
        goto err;
    }

    /* Calculation formula: rotation angle = arctan((coordinate of target in sensor - coordinate of center point in
     * sensor) / hybrid focal length). Here, suppose that images of all pixels of sensor are displayed in screen,
     * and that center of the image sensor coincides with center of rotation of the gimbal, and that optical axis of
     * camera coincides with x-axis of gimbal. */
    hybridFocalLength = s_cameraOpticalZoomFocalLength * s_cameraDigitalZoomFactor;

    psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        errorCode = psdkStat;
        goto err;
    }

    rotationValue.pitch = (int32_t) (
        atan2f((target.focusY - CENTER_POINT_IN_SCREEN_Y_VALUE) * IMAGE_SENSOR_Y_SIZE, hybridFocalLength) * 1800 /
        PI);
    rotationValue.yaw = (int32_t) (
        atan2f((target.focusX - CENTER_POINT_IN_SCREEN_X_VALUE) * IMAGE_SENSOR_X_SIZE, hybridFocalLength) * 1800 /
        PI);

    psdkStat = PsdkOsal_MutexLock(s_tapZoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        errorCode = psdkStat;
        goto err;
    }

    s_tapZoomNewestGimbalRotationArgument.rotationMode = rotationMode;
    s_tapZoomNewestGimbalRotationArgument.rotationProperty = rotationProperty;
    s_tapZoomNewestGimbalRotationArgument.rotationValue = rotationValue;
    s_tapZoomNewestTargetHybridFocalLength = (uint32_t) (hybridFocalLength * (float) s_tapZoomMultiplier);
    s_isTapZooming = true;

    // ensure current value of semaphore is 1
    PsdkOsal_SemaphoreTimedWait(s_tapZoomSem, 0);
    psdkStat = PsdkOsal_SemaphorePost(s_tapZoomSem);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("post semaphore of gimbal rotation error: %lld.", psdkStat);
        errorCode = psdkStat;
        goto err;
    }

    psdkStat = PsdkOsal_MutexUnlock(s_tapZoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        errorCode = psdkStat;
        goto err;
    }

    return PSDK_RETURN_CODE_OK;

err:
    s_isTapZooming = false;

    return errorCode;
}

static T_PsdkReturnCode PsdkTest_CameraHybridZoom(uint32_t focalLength)
{
    if (focalLength >= ZOOM_OPTICAL_FOCAL_MIN_LENGTH && focalLength <= ZOOM_OPTICAL_FOCAL_MAX_LENGTH) {
        s_cameraOpticalZoomFocalLength = focalLength;
        s_cameraDigitalZoomFactor = ZOOM_DIGITAL_BASE_FACTOR;
    } else if (focalLength < ZOOM_OPTICAL_FOCAL_MIN_LENGTH) {
        s_cameraOpticalZoomFocalLength = ZOOM_OPTICAL_FOCAL_MIN_LENGTH;
        s_cameraDigitalZoomFactor = ZOOM_DIGITAL_BASE_FACTOR;

        PsdkLogger_UserLogError("focal length %d is smaller than min optical focal length %d.", focalLength,
                                ZOOM_OPTICAL_FOCAL_MIN_LENGTH);
        return PSDK_RETURN_CODE_ERR_OUT_OF_RANGE;
    } else {
        s_cameraOpticalZoomFocalLength = ZOOM_OPTICAL_FOCAL_MAX_LENGTH;
        s_cameraDigitalZoomFactor = (float) focalLength / ZOOM_OPTICAL_FOCAL_MAX_LENGTH;
    }

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode PsdkTest_CameraRotationGimbal(T_TestCameraGimbalRotationArgument gimbalRotationArgument)
{
    T_PsdkReturnCode psdkStat;
    T_PsdkAircraftInfoBaseInfo aircraftBaseInfo = {0};

    psdkStat = PsdkAircraftInfo_GetBaseInfo(&aircraftBaseInfo);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("get aircraft information error: %lld.", psdkStat);
        return psdkStat;
    }

    if (aircraftBaseInfo.psdkAdapterType == PSDK_AIRCRAFT_INFO_PSDK_ADAPTER_TYPE_SKYPORT_V2) {
        psdkStat = PsdkTest_GimbalRotate(gimbalRotationArgument.rotationMode, gimbalRotationArgument.rotationProperty,
                                         gimbalRotationArgument.rotationValue);
        if (psdkStat != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError("rotate gimbal error: %lld.", psdkStat);
            return psdkStat;
        }
    } else if (aircraftBaseInfo.psdkAdapterType == PSDK_AIRCRAFT_INFO_PSDK_ADAPTER_TYPE_XPORT) {
        psdkStat = PsdkXPort_RotateSync(gimbalRotationArgument.rotationMode, gimbalRotationArgument.rotationProperty,
                                        gimbalRotationArgument.rotationValue);
        if (psdkStat != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError("rotate XPort error: %lld.", psdkStat);
            return psdkStat;
        }
    } else {
        PsdkLogger_UserLogError("PSDK adapter type is invalid: %d.", aircraftBaseInfo.psdkAdapterType);
        return PSDK_RETURN_CODE_ERR_UNSUPPORT;
    }

    return PSDK_RETURN_CODE_OK;
}

#ifndef __CC_ARM
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wmissing-noreturn"
#endif

static void *UserCamera_Task(void *arg)
{
    static uint32_t photoCnt = 0;
    static uint32_t cnt = 0;
    T_PsdkReturnCode psdkStat;
    int tempFocalLength = 0;
    uint32_t currentTime = 0;
    bool isStartIntervalPhotoAction = false;

    USER_UTIL_UNUSED(arg);

    while (1) {
        cnt++;
        PsdkOsal_TaskSleepMs(10);

        psdkStat = PsdkOsal_MutexLock(s_commonMutex);
        if (psdkStat != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
            continue;
        }

        s_cameraSDCardState.availableRecordingTimeInSeconds =
            s_cameraSDCardState.remainSpaceInMB / SDCARD_PER_SECONDS_RECORD_SPACE_IN_MB;
        s_cameraSDCardState.availableCaptureCount = s_cameraSDCardState.remainSpaceInMB / SDCARD_PER_PHOTO_SPACE_IN_MB;

        //realize your photo shoot and storage logic here
        if (s_cameraState.shootingState != PSDK_CAMERA_SHOOTING_PHOTO_IDLE && photoCnt++ > 150) {
            s_cameraState.isStoring = false;
            s_cameraState.shootingState = PSDK_CAMERA_SHOOTING_PHOTO_IDLE;
            photoCnt = 0;

            //store the photo after shooting finished
            if (s_cameraShootPhotoMode == PSDK_CAMERA_SHOOT_PHOTO_MODE_SINGLE) {
                s_cameraSDCardState.remainSpaceInMB =
                    s_cameraSDCardState.remainSpaceInMB - SDCARD_PER_PHOTO_SPACE_IN_MB;
            } else if (s_cameraShootPhotoMode == PSDK_CAMERA_SHOOT_PHOTO_MODE_BURST) {
                s_cameraSDCardState.remainSpaceInMB =
                    s_cameraSDCardState.remainSpaceInMB - SDCARD_PER_PHOTO_SPACE_IN_MB * s_cameraBurstCount;
            } else if (s_cameraShootPhotoMode == PSDK_CAMERA_SHOOT_PHOTO_MODE_INTERVAL) {
                s_cameraSDCardState.remainSpaceInMB =
                    s_cameraSDCardState.remainSpaceInMB - SDCARD_PER_PHOTO_SPACE_IN_MB;
            }

            //check the remain space of sdcard
            if (s_cameraSDCardState.remainSpaceInMB > SDCARD_TOTAL_SPACE_IN_MB) {
                s_cameraSDCardState.remainSpaceInMB = 0;
                s_cameraSDCardState.isFull = true;
            }
        }

        psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
        if (psdkStat != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
            continue;
        }

        // 10Hz
        if (cnt % 10 == 0) {
            psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
            if (psdkStat != PSDK_RETURN_CODE_OK) {
                PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
                continue;
            }

            //Add logic here for zoom camera
            if (s_isStartContinuousOpticalZoom == true) {
                tempFocalLength = s_cameraOpticalZoomFocalLength;

                if (s_cameraZoomDirection == PSDK_CAMERA_ZOOM_DIRECTION_IN) {
                    tempFocalLength += ((int) s_cameraZoomSpeed - PSDK_CAMERA_ZOOM_SPEED_SLOWEST + 1) *
                                       ZOOM_OPTICAL_FOCAL_LENGTH_CTRL_STEP;
                } else if (s_cameraZoomDirection == PSDK_CAMERA_ZOOM_DIRECTION_OUT) {
                    tempFocalLength -= ((int) s_cameraZoomSpeed - PSDK_CAMERA_ZOOM_SPEED_SLOWEST + 1) *
                                       ZOOM_OPTICAL_FOCAL_LENGTH_CTRL_STEP;
                }

                if (tempFocalLength > ZOOM_OPTICAL_FOCAL_MAX_LENGTH) {
                    tempFocalLength = ZOOM_OPTICAL_FOCAL_MAX_LENGTH;
                }

                if (tempFocalLength < ZOOM_OPTICAL_FOCAL_MIN_LENGTH) {
                    tempFocalLength = ZOOM_OPTICAL_FOCAL_MIN_LENGTH;
                }

                s_cameraOpticalZoomFocalLength = (uint16_t) tempFocalLength;
            }

            psdkStat = PsdkOsal_MutexLock(s_tapZoomMutex);
            if (psdkStat != PSDK_RETURN_CODE_OK) {
                PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
                goto out;
            }

            if (s_isTapZooming == true) {
                psdkStat = PsdkOsal_GetTimeMs(&currentTime);
                if (psdkStat != PSDK_RETURN_CODE_OK) {
                    PsdkLogger_UserLogError("get start time error: %lld.", psdkStat);
                }

                if ((currentTime - s_tapZoomStartTime) >= TAP_ZOOM_DURATION) {
                    s_cameraTapZoomState.zoomState = PSDK_CAMERA_TAP_ZOOM_STATE_IDLE;
                    s_cameraTapZoomState.isGimbalMoving = false;
                    s_isTapZooming = false;
                }

                psdkStat = PsdkOsal_SemaphoreTimedWait(s_tapZoomSem, 0);
                if (psdkStat == PSDK_RETURN_CODE_OK) {
                    psdkStat = PsdkTest_CameraRotationGimbal(s_tapZoomNewestGimbalRotationArgument);
                    if (psdkStat != PSDK_RETURN_CODE_OK)
                        PsdkLogger_UserLogError("rotate gimbal error: %lld.", psdkStat);
                    else
                        s_cameraTapZoomState.isGimbalMoving = true;

                    psdkStat = PsdkTest_CameraHybridZoom(s_tapZoomNewestTargetHybridFocalLength);
                    if (psdkStat == PSDK_RETURN_CODE_OK) {
                        s_cameraTapZoomState.zoomState = s_tapZoomNewestTargetHybridFocalLength >
                                                         (s_cameraOpticalZoomFocalLength * s_cameraDigitalZoomFactor)
                                                         ? PSDK_CAMERA_TAP_ZOOM_STATE_ZOOM_IN
                                                         : PSDK_CAMERA_TAP_ZOOM_STATE_ZOOM_OUT;
                    } else if (psdkStat == PSDK_RETURN_CODE_ERR_OUT_OF_RANGE) {
                        PsdkLogger_UserLogError("hybrid zoom focal length beyond limit.");
                        s_cameraTapZoomState.zoomState = PSDK_CAMERA_TAP_ZOOM_STATE_ZOOM_LIMITED;
                    } else {
                        PsdkLogger_UserLogError("hybrid zoom error: %lld.", psdkStat);
                    }
                }
            }

            psdkStat = PsdkOsal_MutexUnlock(s_tapZoomMutex);
            if (psdkStat != PSDK_RETURN_CODE_OK) {
                PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
                goto out;
            }

out:
            psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
            if (psdkStat != PSDK_RETURN_CODE_OK) {
                PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
                continue;
            }
        }

        // 1Hz
        if (cnt % 100 == 0) {
            psdkStat = PsdkOsal_MutexLock(s_commonMutex);
            if (psdkStat != PSDK_RETURN_CODE_OK) {
                PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
                continue;
            }

            if (s_cameraState.isRecording) {
                s_cameraState.currentVideoRecordingTimeInSeconds++;
                s_cameraSDCardState.remainSpaceInMB =
                    s_cameraSDCardState.remainSpaceInMB - SDCARD_PER_SECONDS_RECORD_SPACE_IN_MB;
                if (s_cameraSDCardState.remainSpaceInMB > SDCARD_TOTAL_SPACE_IN_MB) {
                    s_cameraSDCardState.remainSpaceInMB = 0;
                    s_cameraSDCardState.isFull = true;
                }
            }

            if (s_cameraState.isShootingIntervalStart == false) {
                isStartIntervalPhotoAction = false;
            }

            if (s_cameraShootPhotoMode == PSDK_CAMERA_SHOOT_PHOTO_MODE_INTERVAL
                && s_cameraState.isShootingIntervalStart == true && s_cameraPhotoTimeIntervalSettings.captureCount > 0
                && s_cameraPhotoTimeIntervalSettings.timeIntervalSeconds > 0) {
                s_cameraState.currentPhotoShootingIntervalTimeInSeconds--;

                if ((s_cameraState.currentPhotoShootingIntervalTimeInSeconds == 0 &&
                     s_cameraState.currentPhotoShootingIntervalCount > 0) ||
                    (s_cameraState.isShootingIntervalStart == true && isStartIntervalPhotoAction == false)) {

                    isStartIntervalPhotoAction = true;

                    s_cameraState.shootingState = PSDK_CAMERA_SHOOTING_INTERVAL_PHOTO;
                    s_cameraState.isStoring = true;
                    s_cameraState.currentPhotoShootingIntervalTimeInSeconds
                        = s_cameraPhotoTimeIntervalSettings.timeIntervalSeconds;
                    if (s_cameraState.currentPhotoShootingIntervalCount < INTERVAL_PHOTOGRAPH_ALWAYS_COUNT) {
                        PsdkLogger_UserLogDebug("interval taking photograph count:%d interval_time:%ds",
                                                (s_cameraPhotoTimeIntervalSettings.captureCount -
                                                 s_cameraState.currentPhotoShootingIntervalCount + 1),
                                                s_cameraPhotoTimeIntervalSettings.timeIntervalSeconds);
                        s_cameraState.currentPhotoShootingIntervalCount--;
                        if (s_cameraState.currentPhotoShootingIntervalCount == 0) {
                            s_cameraState.shootingState = PSDK_CAMERA_SHOOTING_PHOTO_IDLE;
                            s_cameraState.isStoring = false;
                            s_cameraState.isShootingIntervalStart = false;
                        }
                    } else {
                        PsdkLogger_UserLogDebug("interval taking photograph always, interval_time:%ds",
                                                s_cameraPhotoTimeIntervalSettings.timeIntervalSeconds);
                        s_cameraState.currentPhotoShootingIntervalCount--;
                    }
                }
            }

            psdkStat = PsdkOsal_MutexUnlock(s_commonMutex);
            if (psdkStat != PSDK_RETURN_CODE_OK) {
                PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
                continue;
            }
        }
    }
}

#ifndef __CC_ARM
#pragma clang diagnostic pop
#endif

/* Private functions definition-----------------------------------------------*/
T_PsdkReturnCode PsdkTest_CameraInit(void)
{
    T_PsdkReturnCode returnCode;

    returnCode = PsdkOsal_MutexCreate(&s_commonMutex);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("create mutex used to lock tap zoom arguments error: %lld", returnCode);
        return returnCode;
    }

    returnCode = PsdkOsal_MutexCreate(&s_zoomMutex);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("create mutex used to lock tap zoom arguments error: %lld", returnCode);
        return returnCode;
    }

    returnCode = PsdkOsal_SemaphoreCreate(&s_tapZoomSem, 0);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("create semaphore for synchronizing tap zoom command error: %lld", returnCode);
        return returnCode;
    }

    returnCode = PsdkOsal_MutexCreate(&s_tapZoomMutex);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("create mutex used to lock tap zoom arguments error: %lld", returnCode);
        return returnCode;
    }

    returnCode = PsdkPayloadCamera_Init();
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("payload camera init error:%lld", returnCode);
    }

    /* Init the SDcard parameters */
    s_cameraSDCardState.isInserted = true;
    s_cameraSDCardState.totalSpaceInMB = SDCARD_TOTAL_SPACE_IN_MB;
    s_cameraSDCardState.remainSpaceInMB = SDCARD_TOTAL_SPACE_IN_MB;
    s_cameraSDCardState.availableCaptureCount = SDCARD_TOTAL_SPACE_IN_MB / SDCARD_PER_PHOTO_SPACE_IN_MB;
    s_cameraSDCardState.availableRecordingTimeInSeconds =
        SDCARD_TOTAL_SPACE_IN_MB / SDCARD_PER_SECONDS_RECORD_SPACE_IN_MB;

    /* Register the camera common handler */
    s_commonHandler.GetSystemState = GetSystemState;
    s_commonHandler.SetMode = SetMode;
    s_commonHandler.GetMode = GetMode;
    s_commonHandler.StartRecordVideo = StartRecordVideo;
    s_commonHandler.StopRecordVideo = StopRecordVideo;
    s_commonHandler.StartShootPhoto = StartShootPhoto;
    s_commonHandler.StopShootPhoto = StopShootPhoto;
    s_commonHandler.SetShootPhotoMode = SetShootPhotoMode;
    s_commonHandler.GetShootPhotoMode = GetShootPhotoMode;
    s_commonHandler.SetPhotoBurstCount = SetPhotoBurstCount;
    s_commonHandler.GetPhotoBurstCount = GetPhotoBurstCount;
    s_commonHandler.SetPhotoTimeIntervalSettings = SetPhotoTimeIntervalSettings;
    s_commonHandler.GetPhotoTimeIntervalSettings = GetPhotoTimeIntervalSettings;
    s_commonHandler.GetSDCardState = GetSDCardState;
    s_commonHandler.FormatSDCard = FormatSDCard;

    returnCode = PsdkPayloadCamera_RegCommonHandler(&s_commonHandler);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("camera register common handler error:%lld", returnCode);
    }

    /* Register the camera metering handler */
    s_exposureMeteringHandler.SetMeteringMode = SetMeteringMode;
    s_exposureMeteringHandler.GetMeteringMode = GetMeteringMode;
    s_exposureMeteringHandler.SetSpotMeteringTarget = SetSpotMeteringTarget;
    s_exposureMeteringHandler.GetSpotMeteringTarget = GetSpotMeteringTarget;

    returnCode = PsdkPayloadCamera_RegExposureMeteringHandler(&s_exposureMeteringHandler);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("camera register exposure metering handler error:%lld", returnCode);
        return returnCode;
    }

    /* Register the camera focus handler */
    s_focusHandler.SetFocusMode = SetFocusMode;
    s_focusHandler.GetFocusMode = GetFocusMode;
    s_focusHandler.SetFocusTarget = SetFocusTarget;
    s_focusHandler.GetFocusTarget = GetFocusTarget;
    s_focusHandler.SetFocusAssistantSettings = SetFocusAssistantSettings;
    s_focusHandler.GetFocusAssistantSettings = GetFocusAssistantSettings;
    s_focusHandler.SetFocusRingValue = SetFocusRingValue;
    s_focusHandler.GetFocusRingValue = GetFocusRingValue;
    s_focusHandler.GetFocusRingValueUpperBound = GetFocusRingValueUpperBound;

    returnCode = PsdkPayloadCamera_RegFocusHandler(&s_focusHandler);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("camera register adjustable focal point handler error:%lld", returnCode);
        return returnCode;
    }

    /* Register the camera digital zoom handler */
    s_digitalZoomHandler.SetDigitalZoomFactor = SetDigitalZoomFactor;
    s_digitalZoomHandler.GetDigitalZoomFactor = PsdkTest_CameraGetDigitalZoomFactor;

    returnCode = PsdkPayloadCamera_RegDigitalZoomHandler(&s_digitalZoomHandler);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("camera register digital zoom handler error:%lld", returnCode);
        return returnCode;
    }

    /* Register the camera optical zoom handler */
    s_opticalZoomHandler.SetOpticalZoomFocalLength = SetOpticalZoomFocalLength;
    s_opticalZoomHandler.GetOpticalZoomFocalLength = GetOpticalZoomFocalLength;
    s_opticalZoomHandler.GetOpticalZoomFactor = PsdkTest_CameraGetOpticalZoomFactor;
    s_opticalZoomHandler.GetOpticalZoomSpec = GetOpticalZoomSpec;
    s_opticalZoomHandler.StartContinuousOpticalZoom = StartContinuousOpticalZoom;
    s_opticalZoomHandler.StopContinuousOpticalZoom = StopContinuousOpticalZoom;

    returnCode = PsdkPayloadCamera_RegOpticalZoomHandler(&s_opticalZoomHandler);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("camera register optical zoom handler error:%lld", returnCode);
        return returnCode;
    }

    /* Register the camera tap zoom handler */
    s_tapZoomHandler.GetTapZoomState = GetTapZoomState;
    s_tapZoomHandler.SetTapZoomEnabled = SetTapZoomEnabled;
    s_tapZoomHandler.GetTapZoomEnabled = GetTapZoomEnabled;
    s_tapZoomHandler.SetTapZoomMultiplier = SetTapZoomMultiplier;
    s_tapZoomHandler.GetTapZoomMultiplier = GetTapZoomMultiplier;
    s_tapZoomHandler.TapZoomAtTarget = TapZoomAtTarget;

    returnCode = PsdkPayloadCamera_RegTapZoomHandler(&s_tapZoomHandler);
    if (returnCode != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("camera register tap zoom handler error:%lld", returnCode);
        return returnCode;
    }

    /* Create the camera emu task */
    if (PsdkOsal_TaskCreate(&s_userCameraThread, UserCamera_Task, "user_camera_task",
                            PAYLOAD_CAMERA_EMU_TASK_STACK_SIZE, NULL)
        != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("user camera task create error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    return PSDK_RETURN_CODE_OK;
}

T_PsdkReturnCode PsdkTest_CameraGetDigitalZoomFactor(psdk_f32_t *factor)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    *factor = s_cameraDigitalZoomFactor;

    psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

T_PsdkReturnCode PsdkTest_CameraGetOpticalZoomFactor(psdk_f32_t *factor)
{
    T_PsdkReturnCode psdkStat;

    psdkStat = PsdkOsal_MutexLock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("lock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    //Formula:factor = currentFocalLength / minFocalLength
    *factor = (psdk_f32_t) s_cameraOpticalZoomFocalLength / ZOOM_OPTICAL_FOCAL_MIN_LENGTH;

    psdkStat = PsdkOsal_MutexUnlock(s_zoomMutex);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("unlock mutex error: %lld.", psdkStat);
        return psdkStat;
    }

    return PSDK_RETURN_CODE_OK;
}

/****************** (C) COPYRIGHT DJI Innovations *****END OF FILE****/
