/**
 ********************************************************************
 * @file    test_payload_collaboration.c
 * @version V2.0.0
 * @date    2019/9/3
 * @brief
 *
 * @copyright (c) 2018-2019 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 *********************************************************************
 */


/* Includes ------------------------------------------------------------------*/
#include "test_payload_collaboration.h"
#include "psdk_payload_collaboration.h"
#include "psdk_logger.h"
#include "psdk_platform.h"
#include "utils/util_misc.h"

/* Private constants ---------------------------------------------------------*/
#define PSDK_TEST_PAYLOAD_COLLABORATION_TASK_FREQ       (1)
#define PSDK_TEST_PAYLOAD_COLLABORATION_TASK_STACK_SIZE (2048)

/* Private types -------------------------------------------------------------*/


/* Private functions declaration ---------------------------------------------*/
static void *PsdkTest_PayloadCollaborationTask(void *arg);

/* Private variables ---------------------------------------------------------*/
static T_PsdkTaskHandle s_payloadCollaborationThread;

/* Exported functions definition ---------------------------------------------*/
T_PsdkReturnCode PsdkTest_PayloadCollaborationInit(void)
{
    T_PsdkReturnCode psdkStat;
    T_PsdkAircraftInfoBaseInfo aircraftBaseInfo = {0};

    if (PsdkPayloadCollaboration_Init() != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("payload collaboration module init error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    psdkStat = PsdkAircraftInfo_GetBaseInfo(&aircraftBaseInfo);
    if (psdkStat != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("get aircraft information error: %lld.", psdkStat);
        return psdkStat;
    }

    if (aircraftBaseInfo.payloadMountPosition == PSDK_AIRCRAFT_INFO_PAYLOAD_MOUNT_POSITION_NO1) {
        PsdkLogger_UserLogError("do not meet conditions of payload collaboration.");
        return PSDK_RETURN_CODE_ERR_SYSTEM;
    }

    if (PsdkOsal_TaskCreate(&s_payloadCollaborationThread, PsdkTest_PayloadCollaborationTask,
                            "user_payload_collaboration_task", PSDK_TEST_PAYLOAD_COLLABORATION_TASK_STACK_SIZE, NULL) !=
        PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("user payload collaboration task create error.");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    return PSDK_RETURN_CODE_OK;
}

/* Private functions definition-----------------------------------------------*/
#ifndef __CC_ARM
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wmissing-noreturn"
#endif

static void *PsdkTest_PayloadCollaborationTask(void *arg)
{
    T_PsdkReturnCode psdkStat;
    E_PsdkPayloadCollaborationCameraType no1CameraType = PSDK_PAYLOAD_COLLABORATION_CAMERA_TYPE_UNKNOWN;
    T_PsdkCameraOpticalZoomSpec no1CameraOpticalZoomSpec = {0};
    uint16_t no1CameraHybridZoomFocalLength = 0;

    USER_UTIL_UNUSED(arg);

    while (1) {
        PsdkOsal_TaskSleepMs(1000 / PSDK_TEST_PAYLOAD_COLLABORATION_TASK_FREQ);

        psdkStat = PsdkPayloadCollaboration_GetCameraTypeOfPayload(PSDK_AIRCRAFT_INFO_PAYLOAD_MOUNT_POSITION_NO1,
                                                                   &no1CameraType);
        if (psdkStat != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError("get camera type of payload mounted on NO.1 gimbal connector error.");
            continue;
        }
        PsdkLogger_UserLogDebug("camera type of payload mounted on NO.1 gimbal connector is %d.", no1CameraType);

        psdkStat = PsdkPayloadCollaboration_GetCameraOpticalZoomSpecOfPayload(
            PSDK_AIRCRAFT_INFO_PAYLOAD_MOUNT_POSITION_NO1, &no1CameraOpticalZoomSpec);
        if (psdkStat != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError(
                "get camera optical zoom specification of payload mounted on NO.1 gimbal connector error.");
            continue;
        }
        PsdkLogger_UserLogDebug(
            "camera optical zoom spec, maxFocalLength: %d, minFocalLength: %d, focalLengthStep: %d.",
            no1CameraOpticalZoomSpec.maxFocalLength, no1CameraOpticalZoomSpec.minFocalLength,
            no1CameraOpticalZoomSpec.focalLengthStep);

        psdkStat = PsdkPayloadCollaboration_GetCameraHybridZoomFocalLengthOfPayload(
            PSDK_AIRCRAFT_INFO_PAYLOAD_MOUNT_POSITION_NO1, &no1CameraHybridZoomFocalLength);
        if (psdkStat != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError(
                "get camera hybrid zoom focal length of payload mounted on NO.1 gimbal connector error.");
            continue;
        }
        PsdkLogger_UserLogDebug("camera hybrid zoom focal length, focalLength: %d.", no1CameraHybridZoomFocalLength);
    }
}

#ifndef __CC_ARM
#pragma clang diagnostic pop
#endif

/****************** (C) COPYRIGHT DJI Innovations *****END OF FILE****/
