/**
 ********************************************************************
 * @file    main.c
 * @version V2.0.0
 * @date    2019/07/01
 * @brief
 *
 * @copyright (c) 2018-2019 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 *********************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include <fcntl.h>
#include <zconf.h>
#include <stdio.h>

#include "psdk_logger.h"
#include "psdk_core.h"
#include "psdk_platform.h"

#include "osal/osal.h"
#include "utils/util_misc.h"
#include "hal/hal_uart.h"
#include "psdk_config.h"
#include "app_info.h"
#include "psdk_aircraft_info.h"
#include "camera_emu/test_payload_cam_emu.h"
#include "data_transmission/test_data_transmission.h"
#include "data_subscription/test_data_subscription.h"
#include "payload_collaboration/test_payload_collaboration.h"
#include "camera_media_emu/test_payload_cam_media.h"
#include "widget/test_widget.h"
#include "gimbal_emu/test_payload_gimbal_emu.h"
#include "psdk_data_channel.h"
#include "data_channel/test_data_channel.h"
#include "power_management/test_power_management.h"
#include "xport/test_xport.h"
#include "performance_monitor.h"
#include "psdk_product_info.h"

/* Private constants ---------------------------------------------------------*/
#define PSDK_LOG_PATH             "your_local_log_path"
#define PSDK_LOG_PATH_MAX_SIZE    (64)
#define CMD_BUF_SIZE              (512)

/* Private types -------------------------------------------------------------*/
typedef struct {
    pid_t tid;
    char name[16];
    float pcpu;
} T_ThreadAttribute;

/* Private functions declaration ---------------------------------------------*/
static T_PsdkReturnCode PsdkUser_FillInUserInfo(T_PsdkUserInfo *userInfo);
static T_PsdkReturnCode PsdkUser_Console(const uint8_t *data, uint16_t dataLen);
static T_PsdkReturnCode PsdkUser_LocalWrite(const uint8_t *data, uint16_t dataLen);
static T_PsdkReturnCode PsdkUser_FileSystemInit(const char *path);
static void *PsdkUser_MonitorTask(void *argument);

/* Exported functions definition ---------------------------------------------*/
static int s_psdkFileFd = -1;
static pthread_t s_monitorThread = 0;

/* Private functions definition-----------------------------------------------*/
static T_PsdkReturnCode PsdkUser_FillInUserInfo(T_PsdkUserInfo *userInfo)
{
    memset(userInfo->appName, 0, sizeof(userInfo->appName));
    memset(userInfo->appId, 0, sizeof(userInfo->appId));
    memset(userInfo->appKey, 0, sizeof(userInfo->appKey));
    memset(userInfo->developerAccount, 0, sizeof(userInfo->developerAccount));

    if (strlen(USER_APP_NAME) >= sizeof(userInfo->appName) ||
        strlen(USER_APP_ID) > sizeof(userInfo->appId) ||
        strlen(USER_APP_KEY) > sizeof(userInfo->appKey) ||
        strlen(USER_DEVELOPER_ACCOUNT) >= sizeof(userInfo->developerAccount)) {
        PsdkLogger_UserLogError("Length of user information string is beyond limit. Please check.");
        return PSDK_RETURN_CODE_ERR_PARAM;
    }

    if (!strcmp(USER_APP_NAME, "your_app_name") ||
        !strcmp(USER_APP_ID, "your_app_id") ||
        !strcmp(USER_APP_KEY, "your_app_key") ||
        !strcmp(USER_DEVELOPER_ACCOUNT, "your_developer_account")) {
        PsdkLogger_UserLogError("Please fill in correct user information to psdk_app_info.h file.");
        return PSDK_RETURN_CODE_ERR_PARAM;
    }

    strncpy(userInfo->appName, USER_APP_NAME, sizeof(userInfo->appName) - 1);
    memcpy(userInfo->appId, USER_APP_ID, USER_UTIL_MIN(sizeof(userInfo->appId), strlen(USER_APP_ID)));
    memcpy(userInfo->appKey, USER_APP_KEY, USER_UTIL_MIN(sizeof(userInfo->appKey), strlen(USER_APP_KEY)));
    strncpy(userInfo->developerAccount, USER_DEVELOPER_ACCOUNT, sizeof(userInfo->developerAccount) - 1);

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode PsdkUser_Console(const uint8_t *data, uint16_t dataLen)
{
    USER_UTIL_UNUSED(dataLen);

    printf("%s", data);

    return PSDK_RETURN_CODE_OK;
}

static T_PsdkReturnCode PsdkUser_LocalWrite(const uint8_t *data, uint16_t dataLen)
{
    int32_t realLen;

    if (s_psdkFileFd == -1) {
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    realLen = write(s_psdkFileFd, data, dataLen);
    if (realLen == dataLen) {
        return PSDK_RETURN_CODE_OK;
    } else {
        PsdkLogger_UserLogError("filewrite send error");

        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }
}

static T_PsdkReturnCode PsdkUser_FileSystemInit(const char *path)
{
    T_PsdkReturnCode psdkStat = PSDK_RETURN_CODE_OK;
    char filePath[PSDK_LOG_PATH_MAX_SIZE];

    sprintf(filePath, "%s_%s_%s.log", path, __DATE__, __TIME__);
    s_psdkFileFd = open(filePath, O_RDWR | O_CREAT | O_APPEND,
                        S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
    if (s_psdkFileFd == -1) {
        psdkStat = PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    return psdkStat;
}

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wmissing-noreturn"

int main(void)
{
    T_PsdkUserInfo userInfo;
    const T_PsdkDataChannelBandwidthProportionOfHighspeedChannel bandwidthProportionOfHighspeedChannel =
        {10, 60, 30};
    T_PsdkLoggerConsole printConsole = {
        .consoleLevel = PSDK_LOGGER_CONSOLE_LOG_LEVEL_DEBUG,
        .func = PsdkUser_Console,
    };

    T_PsdkLoggerConsole localRecordConsole = {
        .consoleLevel = PSDK_LOGGER_CONSOLE_LOG_LEVEL_INFO,
        .func = PsdkUser_LocalWrite,
    };

    T_PsdkHalUartHandler halUartHandler = {
        .UartInit = Hal_UartInit,
        .UartReadData = Hal_UartReadData,
        .UartWriteData = Hal_UartSendData,
    };

    T_PsdkOsalHandler osalHandler = {
        .Malloc = Osal_Malloc,
        .Free = Osal_Free,
        .TaskCreate = Osal_TaskCreate,
        .TaskDestroy = Osal_TaskDestroy,
        .TaskSleepMs = Osal_TaskSleepMs,
        .MutexCreate = Osal_MutexCreate,
        .MutexDestroy = Osal_MutexDestroy,
        .MutexLock = Osal_MutexLock,
        .MutexUnlock = Osal_MutexUnlock,
        .SemaphoreCreate = Osal_SemaphoreCreate,
        .SemaphoreDestroy = Osal_SemaphoreDestroy,
        .SemaphoreWait = Osal_SemaphoreWait,
        .SemaphorePost = Osal_SemaphorePost,
        .SemaphoreTimedWait = Osal_SemaphoreTimedWait,
        .GetTimeMs = Osal_GetTimeMs,
    };

    T_PsdkAircraftInfoBaseInfo aircraftBaseInfo = {0};

    if (PsdkPlatform_RegHalUartHandler(&halUartHandler) != PSDK_RETURN_CODE_OK) {
        printf("psdk register hal uart handler error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    if (PsdkPlatform_RegOsalHandler(&osalHandler) != PSDK_RETURN_CODE_OK) {
        printf("psdk register osal handler error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    if (PsdkLogger_AddConsole(&printConsole) != PSDK_RETURN_CODE_OK) {
        printf("psdk add console print error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    if (PsdkUser_FileSystemInit(PSDK_LOG_PATH) != PSDK_RETURN_CODE_OK) {
        printf("psdk file system init error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    if (PsdkLogger_AddConsole(&localRecordConsole) != PSDK_RETURN_CODE_OK) {
        printf("psdk add console record error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    if (PsdkUser_FillInUserInfo(&userInfo) != PSDK_RETURN_CODE_OK) {
        printf("psdk fill in user info error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    if (PsdkCore_Init(&userInfo) != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("psdk instance init error");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    if (PsdkProductInfo_SetAlias("PSDK_APPALIAS") != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("set product alias error.");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    if (PsdkAircraftInfo_GetBaseInfo(&aircraftBaseInfo) != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("get aircraft information error.");
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

#ifdef PSDK_USING_POWER_MANAGEMENT
    if (PsdkTest_PowerManagementInit() != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("power management init error");
    }
#endif

#ifdef PSDK_USING_CAMERA_EMU
    if (PsdkTest_CameraInit() != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("psdk camera init error");
    }
#endif

#ifdef PSDK_USING_CAMERA_MEDIA_EMU
#ifdef PSDK_ARCH_SYS_LINUX
    if (PsdkTest_CameraMediaInit() != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("psdk camera media init error");
    }
#endif
#endif

#ifdef PSDK_USING_DATA_TRANSMISSION
    if (PsdkTest_DataTransmissionInit() != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("psdk data transmission init error");
    }
#endif

#ifdef PSDK_USING_DATA_CHANNEL
    if (PsdkTest_DataChannelSetBandwidthProportionForHighspeedChannel(
        bandwidthProportionOfHighspeedChannel) != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("set bandwidth distribution for high-speed channel error");
    }
#endif

#ifdef PSDK_USING_DATA_SUBSCRIPTION
    if (PsdkTest_DataSubscriptionInit() != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("psdk data subscription init error");
    }
#endif

#ifdef PSDK_USING_PAYLOAD_COLLABORATION
    if (aircraftBaseInfo.payloadMountPosition == PSDK_AIRCRAFT_INFO_PAYLOAD_MOUNT_POSITION_NO2) {
        if (PsdkTest_PayloadCollaborationInit() != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError("psdk payload collaboration init error");
        }
    }
#endif

#ifdef PSDK_USING_WIDGET
    if (PsdkTest_WidgetInit() != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("psdk widget init error");
    }
#endif

#ifdef PSDK_USING_GIMBAL_EMU
    if (aircraftBaseInfo.psdkAdapterType == PSDK_AIRCRAFT_INFO_PSDK_ADAPTER_TYPE_SKYPORT_V2) {
        if (PsdkTest_GimbalInit() != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError("psdk gimbal init error");
        }
    }
#endif

#ifdef PSDK_USING_XPORT
    if (aircraftBaseInfo.psdkAdapterType == PSDK_AIRCRAFT_INFO_PSDK_ADAPTER_TYPE_XPORT) {
        if (PsdkTest_XPortInit() != PSDK_RETURN_CODE_OK) {
            PsdkLogger_UserLogError("psdk XPort init error");
        }
    }
#endif

    if (pthread_create(&s_monitorThread, NULL, PsdkUser_MonitorTask, NULL) != 0) {
        PsdkLogger_UserLogError("create monitor task fail.");
    }

    if (pthread_setname_np(s_monitorThread, "monitor task") != 0) {
        PsdkLogger_UserLogError("set name for monitor task fail.");
    }

    if (PsdkCore_ApplicationStart() != PSDK_RETURN_CODE_OK) {
        PsdkLogger_UserLogError("psdk application start error");
    }

    while (1) {
        sleep(1);
    }
}

static void *PsdkUser_MonitorTask(void *argument)
{
    unsigned int i = 0;
    unsigned int threadCount = 0;
    pid_t *tidList = NULL;
    T_ThreadAttribute *threadAttribute = NULL;

    USER_UTIL_UNUSED(argument);

    while (1) {
        threadCount = getThreadCountOfProcess(getpid());
        tidList = PsdkOsal_Malloc(threadCount * sizeof(pid_t));
        if (tidList == NULL) {
            PsdkLogger_UserLogError("malloc fail.");
            continue;
        }
        getTidListOfProcess(getpid(), tidList, threadCount);

        threadAttribute = PsdkOsal_Malloc(threadCount * sizeof(T_ThreadAttribute));
        if (threadAttribute == NULL) {
            PsdkLogger_UserLogError("malloc fail.");
            continue;
        }
        for (i = 0; i < threadCount; ++i) {
            threadAttribute[i].tid = tidList[i];
        }

        PsdkLogger_UserLogInfo("thread pcpu:");
        PsdkLogger_UserLogInfo("tid\tname\tpcpu");
        for (i = 0; i < threadCount; ++i) {
            threadAttribute[i].pcpu = getPcpuOfThread(getpid(), tidList[i]);
            getNameOfThread(getpid(), tidList[i], threadAttribute[i].name, sizeof(threadAttribute[i].name));
            PsdkLogger_UserLogInfo("%d\t%15s\t%f %%.", threadAttribute[i].tid, threadAttribute[i].name,
                                   threadAttribute[i].pcpu);
        }

        PsdkLogger_UserLogInfo("heap used: %d B.", getHeapUsed(getpid()));
        PsdkLogger_UserLogInfo("stack used: %d B.", getStackUsed(getpid()));

        sleep(10);
    }
}

#pragma clang diagnostic pop

/****************** (C) COPYRIGHT DJI Innovations *****END OF FILE****/
